@@ -3,8 +3,10 @@ use base 'Maypole';
 
 use strict;
 use warnings;
+use CGI::Simple;
+use Maypole::Headers;
 
-our $VERSION = '2.04';
+our $VERSION = '2.05';
 
 sub run {
     my $self = shift;
@@ -12,15 +14,25 @@ sub run {
 }
 
 sub get_request {
-    require CGI::Simple;
     shift->{cgi} = CGI::Simple->new();
 }
 
+
 sub parse_location {
     my $self = shift;
-    $self->{path} = $self->{cgi}->url( -absolute => 1, -path_info => 1 );
-    my $loc = $self->{cgi}->url( -absolute => 1 );
+    my $cgi = $self->{cgi};
+
+    # Reconstruct the request headers (as far as this is possible)
+    $self->headers_in(Maypole::Headers->new);
+    for my $http_header ($cgi->http) {
+        (my $field_name = $http_header) =~ s/^HTTPS?_//;
+        $self->headers_in->set($field_name => $cgi->http($http_header));
+    }
+
+    $self->{path} = $cgi->url( -absolute => 1, -path_info => 1 );
+    my $loc = $cgi->url( -absolute => 1 );
     no warnings 'uninitialized';
+    $self->{path} .= '/' if $self->{path} eq $loc;
     $self->{path} =~ s/^($loc)?\///;
     $self->parse_path;
     $self->parse_args;
@@ -39,12 +51,19 @@ sub parse_args {
 
 sub send_output {
     my $r = shift;
-    print $r->{cgi}->header(
-        -type           => $r->{content_type},
-        -charset        => $r->{document_encoding},
-        -content_length => length $r->{output},
+
+    # Collect HTTP headers
+    my %headers = (
+        -type            => $r->{content_type},
+        -charset         => $r->{document_encoding},
+        -content_length  => do { use bytes; length $r->{output} },
     );
-    print $r->{output};
+    foreach ($r->headers_out->field_names) {
+        next if /^Content-(Type|Length)/;
+        $headers{"-$_"} = $r->headers_out->get($_);
+    }
+
+    print $r->{cgi}->header(%headers), $r->{output};
 }
 
 sub get_template_root {
@@ -61,10 +80,11 @@ CGI::Maypole - CGI-based front-end to Maypole
 =head1 SYNOPSIS
 
      package BeerDB;
-     use base 'CGI::Maypole;
+     use base 'CGI::Maypole';
      BeerDB->setup("dbi:mysql:beerdb");
      BeerDB->config->uri_base("http://your.site/cgi-bin/beer.cgi/");
      BeerDB->config->display_tables([qw[beer brewery pub style]]);
+     BeerDB->config->template_root("/var/www/beerdb/");
      # Now set up your database:
      # has-a relationships
      # untaint columns
@@ -72,16 +92,21 @@ CGI::Maypole - CGI-based front-end to Maypole
      1;
 
      ## example beer.cgi:
-	
+
      #!/usr/bin/perl -w
      use strict;
      use BeerDB;
      BeerDB->run();
 
+Now to access the beer database, type this URL into your browser:
+http://your.site/cgi-bin/beer.cgi/frontpage
+
 =head1 DESCRIPTION
 
-This is a handler for Maypole which will use the CGI instead of Apache's
-C<mod_perl> 1.x. This handler can also be used for Apache 2.0.
+This is a CGI platform driver for Maypole. Your application can inherit from
+CGI::Maypole directly, but it is recommended that you use
+L<Maypole::Application>.
+
 
 =head1 METHODS
 
@@ -95,7 +120,7 @@ Call this from your CGI script to start the Maypole application.
 
 =head1 Implementation
 
-This class overrides a set of methods in the base Maypole class to provide it's 
+This class overrides a set of methods in the base Maypole class to provide it's
 functionality. See L<Maypole> for these:
 
 =over
@@ -117,3 +142,5 @@ functionality. See L<Maypole> for these:
 Dave Ranney C<dave@sialia.com>
 
 Simon Cozens C<simon@cpan.org>
+
+=cut
